"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProgramProxy = void 0;
const ts = require("typescript/lib/tsserverlibrary");
const apis = require("./apis");
const vue_typescript_1 = require("@volar/vue-typescript");
const vue_typescript_2 = require("@volar/vue-typescript");
function createProgramProxy(options, // rootNamesOrOptions: readonly string[] | CreateProgramOptions,
_options, _host, _oldProgram, _configFileParsingDiagnostics) {
    if (!options.options.noEmit && !options.options.emitDeclarationOnly)
        return doThrow('js emit is not support');
    if (!options.host)
        return doThrow('!options.host');
    const host = options.host;
    const vueCompilerOptions = getVueCompilerOptions();
    const scripts = new Map();
    const vueLsHost = {
        ...host,
        resolveModuleNames: undefined,
        writeFile: undefined,
        getCompilationSettings: () => options.options,
        getVueCompilationSettings: () => vueCompilerOptions,
        getScriptFileNames: () => options.rootNames,
        getScriptVersion: (fileName) => scripts.get(fileName)?.version ?? '',
        getScriptSnapshot,
        getProjectVersion: () => '',
        getVueProjectVersion: () => '',
        getProjectReferences: () => options.projectReferences,
    };
    const tsRuntime = (0, vue_typescript_1.createTypeScriptRuntime)({
        typescript: ts,
        baseCssModuleType: 'any',
        getCssClasses: () => ({}),
        vueCompilerOptions,
        vueLsHost: vueLsHost,
        isVueTsc: true,
    });
    const tsProgram = tsRuntime.getTsLs('script').getProgram();
    if (!tsProgram)
        throw '!tsProgram';
    const tsProgramApis_2 = apis.register(ts, tsRuntime);
    const tsProgramProxy = new Proxy(tsProgram, {
        get: (target, property) => {
            tsRuntime.update(true);
            return tsProgramApis_2[property] || target[property];
        },
    });
    for (const rootName of options.rootNames) {
        // register file watchers
        host.getSourceFile(rootName, ts.ScriptTarget.ESNext);
    }
    return tsProgramProxy;
    function getVueCompilerOptions() {
        const tsConfig = options.options.configFilePath;
        if (typeof tsConfig === 'string') {
            return vue_typescript_2.tsShared.createParsedCommandLine(ts, ts.sys, tsConfig).vueOptions;
        }
        return {};
    }
    function getScriptSnapshot(fileName) {
        const script = scripts.get(fileName);
        if (script) {
            return script.scriptSnapshot;
        }
        if (host.fileExists(fileName)) {
            const fileContent = host.readFile(fileName);
            if (fileContent !== undefined) {
                const scriptSnapshot = ts.ScriptSnapshot.fromString(fileContent);
                scripts.set(fileName, {
                    scriptSnapshot: scriptSnapshot,
                    version: ts.sys.createHash?.(fileContent) ?? fileContent,
                });
                return scriptSnapshot;
            }
        }
    }
}
exports.createProgramProxy = createProgramProxy;
function doThrow(msg) {
    console.error(msg);
    throw msg;
}
//# sourceMappingURL=proxy.js.map